@@ -119,7 +119,7 @@ g_mime_cipher_context_class_init (GMimeCipherContextClass *klass)
 static void
 g_mime_cipher_context_init (GMimeCipherContext *ctx, GMimeCipherContextClass *klass)
 {
-	ctx->session = NULL;
+	ctx->request_passwd = NULL;
 }
 
 static void
@@ -127,9 +127,6 @@ g_mime_cipher_context_finalize (GObject *object)
 {
 	GMimeCipherContext *ctx = (GMimeCipherContext *) object;
 	
-	if (ctx->session)
-		g_object_unref (ctx->session);
-	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -141,6 +138,41 @@ cipher_hash_id (GMimeCipherContext *ctx, const char *hash)
 }
 
 
+/**
+ * g_mime_cipher_context_set_request_password:
+ * @ctx: a #GMimeCipherContext
+ * @request_passwd: a callback function for requesting a password
+ *
+ * Sets the function used by the @ctx for requesting a password from
+ * the user.
+ **/
+void
+g_mime_cipher_context_set_request_password (GMimeCipherContext *ctx, GMimePasswordRequestFunc request_passwd)
+{
+	g_return_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx));
+	
+	ctx->request_passwd = request_passwd;
+}
+
+
+/**
+ * g_mime_cipher_context_get_request_password:
+ * @ctx: a #GMimeCipherContext
+ *
+ * Gets the function used by the @ctx for requesting a password from
+ * the user.
+ *
+ * Returns: a #GMimePasswordRequestFunc or %NULL if not set.
+ **/
+GMimePasswordRequestFunc
+g_mime_cipher_context_get_request_password (GMimeCipherContext *ctx)
+{
+	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
+	
+	return ctx->request_passwd;
+}
+
+
 /**
  * g_mime_cipher_context_hash_id:
  * @ctx: a #GMimeCipherContext
@@ -189,7 +221,7 @@ static int
 cipher_sign (GMimeCipherContext *ctx, const char *userid, GMimeCipherHash hash,
 	     GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NOT_SUPPORTED,
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
 		     "Signing is not supported by this cipher");
 	
 	return -1;
@@ -226,7 +258,7 @@ static GMimeSignatureValidity *
 cipher_verify (GMimeCipherContext *ctx, GMimeCipherHash hash, GMimeStream *istream,
 	       GMimeStream *sigstream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NOT_SUPPORTED,
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
 		     "Verifying is not supported by this cipher");
 	
 	return NULL;
@@ -265,7 +297,7 @@ static int
 cipher_encrypt (GMimeCipherContext *ctx, gboolean sign, const char *userid, GPtrArray *recipients,
 		GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NOT_SUPPORTED,
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
 		     "Encryption is not supported by this cipher");
 	
 	return -1;
@@ -303,7 +335,7 @@ static GMimeSignatureValidity *
 cipher_decrypt (GMimeCipherContext *ctx, GMimeStream *istream,
 		GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NOT_SUPPORTED,
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
 		     "Decryption is not supported by this cipher");
 	
 	return NULL;
@@ -346,7 +378,7 @@ g_mime_cipher_context_decrypt (GMimeCipherContext *ctx, GMimeStream *istream,
 static int
 cipher_import_keys (GMimeCipherContext *ctx, GMimeStream *istream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NOT_SUPPORTED,
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
 		     "You may not import keys with this cipher");
 	
 	return -1;
@@ -378,7 +410,7 @@ static int
 cipher_export_keys (GMimeCipherContext *ctx, GPtrArray *keys,
 		    GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NOT_SUPPORTED,
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
 		     "You may not export keys with this cipher");
 	
 	return -1;
@@ -425,10 +457,15 @@ g_mime_signer_new (void)
 	signer->status = GMIME_SIGNER_STATUS_NONE;
 	signer->errors = GMIME_SIGNER_ERROR_NONE;
 	signer->trust = GMIME_SIGNER_TRUST_NONE;
-	signer->created = (time_t) 0;
-	signer->expires = (time_t) 0;
+	signer->sig_created = (time_t) 0;
+	signer->sig_expires = (time_t) 0;
+	signer->key_created = (time_t) 0;
+	signer->key_expires = (time_t) 0;
+	signer->issuer_serial = NULL;
+	signer->issuer_name = NULL;
 	signer->fingerprint = NULL;
 	signer->keyid = NULL;
+	signer->email = NULL;
 	signer->name = NULL;
 	signer->next = NULL;
 	
@@ -445,8 +482,11 @@ g_mime_signer_new (void)
 void
 g_mime_signer_free (GMimeSigner *signer)
 {
+	g_free (signer->issuer_serial);
+	g_free (signer->issuer_name);
 	g_free (signer->fingerprint);
 	g_free (signer->keyid);
+	g_free (signer->email);
 	g_free (signer->name);
 	
 	g_slice_free (GMimeSigner, signer);
@@ -569,6 +609,74 @@ g_mime_signer_get_trust (GMimeSigner *signer)
 }
 
 
+/**
+ * g_mime_signer_set_issuer_serial:
+ * @signer: a #GMimeSigner
+ * @issuer_serial: signer's issuer serial
+ *
+ * Set the signer's issuer serial.
+ **/
+void
+g_mime_signer_set_issuer_serial (GMimeSigner *signer, const char *issuer_serial)
+{
+	g_return_if_fail (signer != NULL);
+	
+	g_free (signer->issuer_serial);
+	signer->issuer_serial = g_strdup (issuer_serial);
+}
+
+
+/**
+ * g_mime_signer_get_issuer_serial:
+ * @signer: a #GMimeSigner
+ *
+ * Get the signer's issuer serial.
+ *
+ * Returns: the signer's issuer serial.
+ **/
+const char *
+g_mime_signer_get_issuer_serial (GMimeSigner *signer)
+{
+	g_return_val_if_fail (signer != NULL, NULL);
+	
+	return signer->issuer_serial;
+}
+
+
+/**
+ * g_mime_signer_set_issuer_name:
+ * @signer: a #GMimeSigner
+ * @issuer_name: signer's issuer name
+ *
+ * Set the signer's issuer name.
+ **/
+void
+g_mime_signer_set_issuer_name (GMimeSigner *signer, const char *issuer_name)
+{
+	g_return_if_fail (signer != NULL);
+	
+	g_free (signer->issuer_name);
+	signer->issuer_name = g_strdup (issuer_name);
+}
+
+
+/**
+ * g_mime_signer_get_issuer_name:
+ * @signer: a #GMimeSigner
+ *
+ * Get the signer's issuer name.
+ *
+ * Returns: the signer's issuer name.
+ **/
+const char *
+g_mime_signer_get_issuer_name (GMimeSigner *signer)
+{
+	g_return_val_if_fail (signer != NULL, NULL);
+	
+	return signer->issuer_name;
+}
+
+
 /**
  * g_mime_signer_set_fingerprint:
  * @signer: a #GMimeSigner
@@ -637,6 +745,40 @@ g_mime_signer_get_key_id (GMimeSigner *signer)
 }
 
 
+/**
+ * g_mime_signer_set_email:
+ * @signer: a #GMimeSigner
+ * @email: signer's email
+ *
+ * Set the signer's email.
+ **/
+void
+g_mime_signer_set_email (GMimeSigner *signer, const char *email)
+{
+	g_return_if_fail (signer != NULL);
+	
+	g_free (signer->email);
+	signer->email = g_strdup (email);
+}
+
+
+/**
+ * g_mime_signer_get_email:
+ * @signer: a #GMimeSigner
+ *
+ * Get the signer's email.
+ *
+ * Returns: the signer's email.
+ **/
+const char *
+g_mime_signer_get_email (GMimeSigner *signer)
+{
+	g_return_val_if_fail (signer != NULL, NULL);
+	
+	return signer->email;
+}
+
+
 /**
  * g_mime_signer_set_name:
  * @signer: a #GMimeSigner
@@ -658,9 +800,9 @@ g_mime_signer_set_name (GMimeSigner *signer, const char *name)
  * g_mime_signer_get_name:
  * @signer: a #GMimeSigner
  *
- * Get the signer trust.
+ * Get the signer's name.
  *
- * Returns: the signer trust.
+ * Returns: the signer's name.
  **/
 const char *
 g_mime_signer_get_name (GMimeSigner *signer)
@@ -672,23 +814,89 @@ g_mime_signer_get_name (GMimeSigner *signer)
 
 
 /**
- * g_mime_signer_set_created:
+ * g_mime_signer_set_sig_created:
  * @signer: a #GMimeSigner
  * @created: creation date
  *
- * Set the signer's key creation date.
+ * Set the creation date of the signer's signature.
  **/
 void
-g_mime_signer_set_created (GMimeSigner *signer, time_t created)
+g_mime_signer_set_sig_created (GMimeSigner *signer, time_t created)
 {
 	g_return_if_fail (signer != NULL);
 	
-	signer->created = created;
+	signer->sig_created = created;
 }
 
 
 /**
- * g_mime_signer_get_created:
+ * g_mime_signer_get_sig_created:
+ * @signer: a #GMimeSigner
+ *
+ * Get the creation date of the signer's signature.
+ *
+ * Returns: the creation date of the signer's signature.
+ **/
+time_t
+g_mime_signer_get_sig_created (GMimeSigner *signer)
+{
+	g_return_val_if_fail (signer != NULL, (time_t) -1);
+	
+	return signer->sig_created;
+}
+
+
+/**
+ * g_mime_signer_set_sig_expires:
+ * @signer: a #GMimeSigner
+ * @expires: expiration date
+ *
+ * Set the expiration date of the signer's signature.
+ **/
+void
+g_mime_signer_set_sig_expires (GMimeSigner *signer, time_t expires)
+{
+	g_return_if_fail (signer != NULL);
+	
+	signer->sig_expires = expires;
+}
+
+
+/**
+ * g_mime_signer_get_sig_expires:
+ * @signer: a #GMimeSigner
+ *
+ * Get the expiration date of the signer's signature.
+ *
+ * Returns: the expiration date of the signer's signature.
+ **/
+time_t
+g_mime_signer_get_sig_expires (GMimeSigner *signer)
+{
+	g_return_val_if_fail (signer != NULL, (time_t) -1);
+	
+	return signer->sig_expires;
+}
+
+
+/**
+ * g_mime_signer_set_key_created:
+ * @signer: a #GMimeSigner
+ * @created: creation date
+ *
+ * Set the creation date of the signer's key.
+ **/
+void
+g_mime_signer_set_key_created (GMimeSigner *signer, time_t created)
+{
+	g_return_if_fail (signer != NULL);
+	
+	signer->key_created = created;
+}
+
+
+/**
+ * g_mime_signer_get_key_created:
  * @signer: a #GMimeSigner
  *
  * Get the creation date of the signer's key.
@@ -696,32 +904,32 @@ g_mime_signer_set_created (GMimeSigner *signer, time_t created)
  * Returns: the creation date of the signer's key.
  **/
 time_t
-g_mime_signer_get_created (GMimeSigner *signer)
+g_mime_signer_get_key_created (GMimeSigner *signer)
 {
 	g_return_val_if_fail (signer != NULL, (time_t) -1);
 	
-	return signer->created;
+	return signer->key_created;
 }
 
 
 /**
- * g_mime_signer_set_expires:
+ * g_mime_signer_set_key_expires:
  * @signer: a #GMimeSigner
  * @expires: expiration date
  *
- * Set the signer's key expiration date.
+ * Set the expiration date of the signer's key.
  **/
 void
-g_mime_signer_set_expires (GMimeSigner *signer, time_t expires)
+g_mime_signer_set_key_expires (GMimeSigner *signer, time_t expires)
 {
 	g_return_if_fail (signer != NULL);
 	
-	signer->expires = expires;
+	signer->key_expires = expires;
 }
 
 
 /**
- * g_mime_signer_get_expires:
+ * g_mime_signer_get_key_expires:
  * @signer: a #GMimeSigner
  *
  * Get the expiration date of the signer's key.
@@ -729,11 +937,11 @@ g_mime_signer_set_expires (GMimeSigner *signer, time_t expires)
  * Returns: the expiration date of the signer's key.
  **/
 time_t
-g_mime_signer_get_expires (GMimeSigner *signer)
+g_mime_signer_get_key_expires (GMimeSigner *signer)
 {
 	g_return_val_if_fail (signer != NULL, (time_t) -1);
 	
-	return signer->expires;
+	return signer->key_expires;
 }
 
 
@@ -879,7 +1087,7 @@ g_mime_signature_validity_get_signers (const GMimeSignatureValidity *validity)
  * Adds @signer to the list of signers on @validity.
  **/
 void
-g_mime_signature_validity_add_signer  (GMimeSignatureValidity *validity, GMimeSigner *signer)
+g_mime_signature_validity_add_signer (GMimeSignatureValidity *validity, GMimeSigner *signer)
 {
 	GMimeSigner *s;
 	
