@@ -28,7 +28,6 @@
 #include <time.h>
 
 #include <gmime/gmime-stream.h>
-#include <gmime/gmime-session.h>
 
 G_BEGIN_DECLS
 
@@ -46,6 +45,24 @@ typedef struct _GMimeSigner GMimeSigner;
 typedef struct _GMimeSignatureValidity GMimeSignatureValidity;
 
 
+/**
+ * GMimePasswordRequestFunc:
+ * @ctx: the #GMimeCipherContext making the request
+ * @user_id: the user_id of the password being requested
+ * @prompt_ctx: a string containing some helpful context for the prompt
+ * @reprompt: %TRUE if this password request is a reprompt due to a previously bad password response
+ * @response: a stream for the application to write the password to (followed by a newline '\n' character)
+ * @err: a #GError for the callback to set if an error occurs
+ *
+ * A password request callback allowing a #GMimeCipherContext to
+ * prompt the user for a password for a given key.
+ *
+ * Returns: %TRUE on success or %FALSE on error.
+ **/
+typedef gboolean (* GMimePasswordRequestFunc) (GMimeCipherContext *ctx, const char *user_id, const char *prompt_ctx,
+					       gboolean reprompt, GMimeStream *response, GError **err);
+
+
 /**
  * GMimeCipherHash:
  * @GMIME_CIPHER_HASH_DEFAULT: The default hash algorithm.
@@ -80,7 +97,7 @@ typedef enum {
 /**
  * GMimeCipherContext:
  * @parent_object: parent #GObject
- * @session: a #GMimeSession
+ * @request_passwd: a callback for requesting a password
  * @sign_protocol: signature protocol (must be set by subclass)
  * @encrypt_protocol: encryption protocol (must be set by subclass)
  * @key_protocol: key exchange protocol (must be set by subclass)
@@ -90,9 +107,9 @@ typedef enum {
 struct _GMimeCipherContext {
 	GObject parent_object;
 	
-	GMimeSession *session;
+	GMimePasswordRequestFunc request_passwd;
 	
-	/* these must be set by the subclass */
+	/* these must be set by the subclass in the instance_init() */
 	const char *sign_protocol;
 	const char *encrypt_protocol;
 	const char *key_protocol;
@@ -131,6 +148,8 @@ struct _GMimeCipherContextClass {
 
 GType g_mime_cipher_context_get_type (void);
 
+void g_mime_cipher_context_set_request_password (GMimeCipherContext *ctx, GMimePasswordRequestFunc request_passwd);
+
 /* hash routines */
 GMimeCipherHash      g_mime_cipher_context_hash_id (GMimeCipherContext *ctx, const char *hash);
 
@@ -214,10 +233,10 @@ typedef enum {
  **/
 typedef enum {
 	GMIME_SIGNER_ERROR_NONE,
-	GMIME_SIGNER_ERROR_EXPSIG     = (1 << 0),  /* expire signature */
-	GMIME_SIGNER_ERROR_NO_PUBKEY  = (1 << 1),  /* no public key */
-	GMIME_SIGNER_ERROR_EXPKEYSIG  = (1 << 2),  /* expired key */
-	GMIME_SIGNER_ERROR_REVKEYSIG  = (1 << 3)   /* revoked key */
+	GMIME_SIGNER_ERROR_EXPSIG      = (1 << 0),  /* expired signature */
+	GMIME_SIGNER_ERROR_NO_PUBKEY   = (1 << 1),  /* no public key */
+	GMIME_SIGNER_ERROR_EXPKEYSIG   = (1 << 2),  /* expired key */
+	GMIME_SIGNER_ERROR_REVKEYSIG   = (1 << 3)   /* revoked key */
 } GMimeSignerError;
 
 
@@ -228,10 +247,15 @@ typedef enum {
  * @errors: A bitfield of #GMimeSignerError values.
  * @trust: A #GMimeSignerTrust.
  * @unused: Unused expansion bits for future use; ignore this.
- * @created: The creation date of the signature.
- * @expires: The expiration date of the signature.
+ * @issuer_serial: The issuer of the certificate if known.
+ * @issuer_name: The issuer of the certificate if known.
  * @fingerprint: A hex string representing the signer's fingerprint.
+ * @sig_created: The creation date of the signature.
+ * @sig_expires: The expiration date of the signature.
+ * @key_created: The creation date of the signature key.
+ * @key_expires: The expiration date of the signature key.
  * @keyid: The signer's key id.
+ * @email: The email address of the person or entity.
  * @name: The name of the person or entity.
  *
  * A structure containing useful information about a signer.
@@ -241,11 +265,16 @@ struct _GMimeSigner {
 	unsigned int status:2;    /* GMimeSignerStatus */
 	unsigned int errors:4;    /* bitfield of GMimeSignerError's */
 	unsigned int trust:3;     /* GMimeSignerTrust */
-	unsigned int unused:23;   /* unused expansion bits */
+	unsigned int unused:21;   /* unused expansion bits */
+	char *issuer_serial;
+	char *issuer_name;
 	char *fingerprint;
-	time_t created;
-	time_t expires;
+	time_t sig_created;
+	time_t sig_expires;
+	time_t key_created;
+	time_t key_expires;
 	char *keyid;
+	char *email;
 	char *name;
 };
 
@@ -264,21 +293,35 @@ GMimeSignerError g_mime_signer_get_errors (GMimeSigner *signer);
 void g_mime_signer_set_trust (GMimeSigner *signer, GMimeSignerTrust trust);
 GMimeSignerTrust g_mime_signer_get_trust (GMimeSigner *signer);
 
+void g_mime_signer_set_issuer_serial (GMimeSigner *signer, const char *issuer_serial);
+const char *g_mime_signer_get_issuer_serial (GMimeSigner *signer);
+
+void g_mime_signer_set_issuer_name (GMimeSigner *signer, const char *issuer_name);
+const char *g_mime_signer_get_issuer_name (GMimeSigner *signer);
+
 void g_mime_signer_set_fingerprint (GMimeSigner *signer, const char *fingerprint);
 const char *g_mime_signer_get_fingerprint (GMimeSigner *signer);
 
 void g_mime_signer_set_key_id (GMimeSigner *signer, const char *key_id);
 const char *g_mime_signer_get_key_id (GMimeSigner *signer);
 
+void g_mime_signer_set_email (GMimeSigner *signer, const char *email);
+const char *g_mime_signer_get_email (GMimeSigner *signer);
+
 void g_mime_signer_set_name (GMimeSigner *signer, const char *name);
 const char *g_mime_signer_get_name (GMimeSigner *signer);
 
-void g_mime_signer_set_created (GMimeSigner *signer, time_t created);
-time_t g_mime_signer_get_created (GMimeSigner *signer);
+void g_mime_signer_set_sig_created (GMimeSigner *signer, time_t created);
+time_t g_mime_signer_get_sig_created (GMimeSigner *signer);
 
-void g_mime_signer_set_expires (GMimeSigner *signer, time_t expires);
-time_t g_mime_signer_get_expires (GMimeSigner *signer);
+void g_mime_signer_set_sig_expires (GMimeSigner *signer, time_t expires);
+time_t g_mime_signer_get_sig_expires (GMimeSigner *signer);
 
+void g_mime_signer_set_key_created (GMimeSigner *signer, time_t created);
+time_t g_mime_signer_get_key_created (GMimeSigner *signer);
+
+void g_mime_signer_set_key_expires (GMimeSigner *signer, time_t expires);
+time_t g_mime_signer_get_key_expires (GMimeSigner *signer);
 
 
 /**
