@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -23,12 +23,14 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <time.h>
 
 #include "gmime-multipart.h"
 #include "gmime-utils.h"
@@ -61,6 +63,7 @@ static gboolean multipart_remove_header (GMimeObject *object, const char *header
 static void multipart_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 static char *multipart_get_headers (GMimeObject *object);
 static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void multipart_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 /* GMimeMultipart class methods */
 static void multipart_clear (GMimeMultipart *multipart);
@@ -122,6 +125,7 @@ g_mime_multipart_class_init (GMimeMultipartClass *klass)
 	object_class->set_content_type = multipart_set_content_type;
 	object_class->get_headers = multipart_get_headers;
 	object_class->write_to_stream = multipart_write_to_stream;
+	object_class->encode = multipart_encode;
 	
 	klass->add = multipart_add;
 	klass->clear = multipart_clear;
@@ -240,7 +244,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	 * not, then it's a broken multipart and so we don't want to
 	 * alter it or we'll completely break the output) */
 	boundary = g_mime_object_get_content_type_parameter (object, "boundary");
-	if (!boundary && !g_mime_header_list_has_raw (object->headers)) {
+	if (!boundary && !g_mime_header_list_get_stream (object->headers)) {
 		g_mime_multipart_set_boundary (multipart, NULL);
 		boundary = g_mime_object_get_content_type_parameter (object, "boundary");
 	}
@@ -300,6 +304,19 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	return total;
 }
 
+static void
+multipart_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	GMimeMultipart *multipart = (GMimeMultipart *) object;
+	GMimeObject *subpart;
+	int i;
+	
+	for (i = 0; i < g_mime_multipart_get_count (multipart); i++) {
+		subpart = g_mime_multipart_get_part (multipart, i);
+		g_mime_object_encode (subpart, constraint);
+	}
+}
+
 
 /**
  * g_mime_multipart_new:
@@ -316,7 +333,7 @@ g_mime_multipart_new (void)
 	GMimeContentType *content_type;
 	GMimeMultipart *multipart;
 	
-	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL);
+	multipart = g_object_newv (GMIME_TYPE_MULTIPART, 0, NULL);
 	
 	content_type = g_mime_content_type_new ("multipart", "mixed");
 	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
@@ -342,7 +359,7 @@ g_mime_multipart_new_with_subtype (const char *subtype)
 	GMimeContentType *content_type;
 	GMimeMultipart *multipart;
 	
-	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL);
+	multipart = g_object_newv (GMIME_TYPE_MULTIPART, 0, NULL);
 	
 	content_type = g_mime_content_type_new ("multipart", subtype ? subtype : "mixed");
 	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
@@ -423,9 +440,9 @@ g_mime_multipart_get_postface (GMimeMultipart *multipart)
 static void
 multipart_clear (GMimeMultipart *multipart)
 {
-	int i;
+	guint i;
 	
-	for (i = 0; i < (int) multipart->children->len; i++)
+	for (i = 0; i < multipart->children->len; i++)
 		g_object_unref (multipart->children->pdata[i]);
 	
 	g_ptr_array_set_size (multipart->children, 0);
@@ -478,18 +495,20 @@ ptr_array_insert (GPtrArray *array, guint index, gpointer object)
 	unsigned char *dest, *src;
 	guint n;
 	
-	g_ptr_array_set_size (array, array->len + 1);
-	
-	if (index == array->len) {
-		/* need to move items down */
+	if (index < array->len) {
+		/* need to shift some items */
+		g_ptr_array_set_size (array, array->len + 1);
+		
 		dest = ((unsigned char *) array->pdata) + (sizeof (void *) * (index + 1));
 		src = ((unsigned char *) array->pdata) + (sizeof (void *) * index);
 		n = array->len - index - 1;
 		
 		g_memmove (dest, src, (sizeof (void *) * n));
+		array->pdata[index] = object;
+	} else {
+		/* the easy case */
+		g_ptr_array_add (array, object);
 	}
-	
-	array->pdata[index] = object;
 }
 
 static void
@@ -556,7 +575,7 @@ multipart_remove_at (GMimeMultipart *multipart, int index)
 {
 	GMimeObject *part;
 	
-	if (index >= multipart->children->len)
+	if ((guint) index >= multipart->children->len)
 		return NULL;
 	
 	part = multipart->children->pdata[index];
@@ -592,7 +611,7 @@ multipart_get_part (GMimeMultipart *multipart, int index)
 {
 	GMimeObject *part;
 	
-	if (index >= multipart->children->len)
+	if ((guint) index >= multipart->children->len)
 		return NULL;
 	
 	part = multipart->children->pdata[index];
@@ -623,9 +642,9 @@ g_mime_multipart_get_part (GMimeMultipart *multipart, int index)
 static gboolean
 multipart_contains (GMimeMultipart *multipart, GMimeObject *part)
 {
-	int i;
+	guint i;
 	
-	for (i = 0; i < (int) multipart->children->len; i++) {
+	for (i = 0; i < multipart->children->len; i++) {
 		if (part == (GMimeObject *) multipart->children->pdata[i])
 			return TRUE;
 	}
@@ -657,9 +676,9 @@ g_mime_multipart_contains (GMimeMultipart *multipart, GMimeObject *part)
 static int
 multipart_index_of (GMimeMultipart *multipart, GMimeObject *part)
 {
-	int i;
+	guint i;
 	
-	for (i = 0; i < (int) multipart->children->len; i++) {
+	for (i = 0; i < multipart->children->len; i++) {
 		if (part == (GMimeObject *) multipart->children->pdata[i])
 			return i;
 	}
@@ -714,6 +733,7 @@ g_mime_multipart_get_count (GMimeMultipart *multipart)
 static void
 read_random_pool (unsigned char *buffer, size_t bytes)
 {
+#ifdef __unix__
 	size_t nread = 0;
 	ssize_t n;
 	int fd;
@@ -735,6 +755,14 @@ read_random_pool (unsigned char *buffer, size_t bytes)
 	} while (nread < bytes);
 	
 	close (fd);
+#else
+	size_t i;
+	
+	srand ((unsigned int) time (NULL));
+	
+	for (i = 0; i < bytes; i++)
+		buffer[i] = (unsigned char) (rand () % 256);
+#endif
 }
 
 static void
