@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -33,6 +33,7 @@
 #include "gmime-stream-null.h"
 #include "gmime-stream-filter.h"
 #include "gmime-filter-basic.h"
+#include "gmime-filter-best.h"
 #include "gmime-filter-crlf.h"
 #include "gmime-filter-md5.h"
 
@@ -62,6 +63,7 @@ static const char *mime_part_get_header (GMimeObject *object, const char *header
 static gboolean mime_part_remove_header (GMimeObject *object, const char *header);
 static char *mime_part_get_headers (GMimeObject *object);
 static ssize_t mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void mime_part_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 /* GMimePart class methods */
 static void set_content_object (GMimePart *mime_part, GMimeDataWrapper *content);
@@ -112,6 +114,7 @@ g_mime_part_class_init (GMimePartClass *klass)
 	object_class->get_header = mime_part_get_header;
 	object_class->get_headers = mime_part_get_headers;
 	object_class->write_to_stream = mime_part_write_to_stream;
+	object_class->encode = mime_part_encode;
 	
 	klass->set_content_object = set_content_object;
 }
@@ -257,11 +260,6 @@ mime_part_remove_header (GMimeObject *object, const char *header)
 	GMimePart *mime_part = (GMimePart *) object;
 	guint i;
 	
-	/* Make sure that the header is a Content-* header, else it
-	   doesn't belong on a mime part */
-	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
-		return FALSE;
-	
 	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
 		if (!g_ascii_strcasecmp (content_headers[i], header))
 			break;
@@ -399,6 +397,54 @@ mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	return total;
 }
 
+static void
+mime_part_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	GMimePart *part = (GMimePart *) object;
+	GMimeContentEncoding encoding;
+	GMimeStream *stream, *null;
+	GMimeFilter *filter;
+	
+	switch (part->encoding) {
+	case GMIME_CONTENT_ENCODING_DEFAULT:
+		/* Unspecified encoding, we need to figure out the
+		 * best encoding no matter what */
+		break;
+	case GMIME_CONTENT_ENCODING_7BIT:
+		/* This encoding is always safe. */
+		return;
+	case GMIME_CONTENT_ENCODING_8BIT:
+		/* This encoding is safe unless the constraint is 7bit. */
+		if (constraint != GMIME_ENCODING_CONSTRAINT_7BIT)
+			return;
+		break;
+	case GMIME_CONTENT_ENCODING_BINARY:
+		/* This encoding is only safe if the constraint is binary. */
+		if (constraint == GMIME_ENCODING_CONSTRAINT_BINARY)
+			return;
+		break;
+	case GMIME_CONTENT_ENCODING_BASE64:
+	case GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE:
+	case GMIME_CONTENT_ENCODING_UUENCODE:
+		/* These encodings are always safe. */
+		return;
+	}
+	
+	filter = g_mime_filter_best_new (GMIME_FILTER_BEST_ENCODING);
+	
+	null = g_mime_stream_null_new ();
+	stream = g_mime_stream_filter_new (null);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) stream, filter);
+	g_object_unref (null);
+	
+	g_mime_object_write_to_stream (object, stream);
+	g_object_unref (stream);
+	
+	encoding = g_mime_filter_best_encoding ((GMimeFilterBest *) filter, constraint);
+	g_mime_part_set_content_encoding (part, encoding);
+	g_object_unref (filter);
+}
+
 
 /**
  * g_mime_part_new:
@@ -481,7 +527,7 @@ g_mime_part_set_content_description (GMimePart *mime_part, const char *descripti
  * Returns: the content description for the specified mime part.
  **/
 const char *
-g_mime_part_get_content_description (const GMimePart *mime_part)
+g_mime_part_get_content_description (GMimePart *mime_part)
 {
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), NULL);
 	
@@ -735,6 +781,45 @@ g_mime_part_get_content_encoding (GMimePart *mime_part)
 }
 
 
+/**
+ * g_mime_part_get_best_content_encoding:
+ * @mime_part: a #GMimePart object
+ * @constraint: a #GMimeEncodingConstraint
+ *
+ * Calculates the most efficient content encoding for the @mime_part
+ * given the @constraint.
+ *
+ * Returns: the best content encoding for the specified mime part.
+ **/
+GMimeContentEncoding
+g_mime_part_get_best_content_encoding (GMimePart *mime_part, GMimeEncodingConstraint constraint)
+{
+	GMimeStream *filtered, *stream;
+	GMimeContentEncoding encoding;
+	GMimeFilterBest *best;
+	GMimeFilter *filter;
+	
+	g_return_val_if_fail (GMIME_IS_PART (mime_part), GMIME_CONTENT_ENCODING_DEFAULT);
+	
+	stream = g_mime_stream_null_new ();
+	filtered = g_mime_stream_filter_new (stream);
+	g_object_unref (stream);
+	
+	filter = g_mime_filter_best_new (GMIME_FILTER_BEST_ENCODING);
+	g_mime_stream_filter_add ((GMimeStreamFilter *) filtered, filter);
+	best = (GMimeFilterBest *) filter;
+	
+	g_mime_data_wrapper_write_to_stream (mime_part->content, filtered);
+	g_mime_stream_flush (filtered);
+	g_object_unref (filtered);
+	
+	encoding = g_mime_filter_best_encoding (best, constraint);
+	g_object_unref (best);
+	
+	return encoding;
+}
+
+
 /**
  * g_mime_part_set_filename:
  * @mime_part: a #GMimePart object
@@ -767,7 +852,7 @@ g_mime_part_set_filename (GMimePart *mime_part, const char *filename)
  * and if not then checks the "name" parameter in the Content-Type.
  **/
 const char *
-g_mime_part_get_filename (const GMimePart *mime_part)
+g_mime_part_get_filename (GMimePart *mime_part)
 {
 	GMimeObject *object = (GMimeObject *) mime_part;
 	const char *filename = NULL;
@@ -792,6 +877,7 @@ set_content_object (GMimePart *mime_part, GMimeDataWrapper *content)
 	g_object_ref (content);
 }
 
+
 /**
  * g_mime_part_set_content_object:
  * @mime_part: a #GMimePart object
@@ -821,7 +907,7 @@ g_mime_part_set_content_object (GMimePart *mime_part, GMimeDataWrapper *content)
  * Returns: the data-wrapper for the mime part's contents.
  **/
 GMimeDataWrapper *
-g_mime_part_get_content_object (const GMimePart *mime_part)
+g_mime_part_get_content_object (GMimePart *mime_part)
 {
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), NULL);
 	
