@@ -52,7 +52,7 @@ static gboolean stream_eos (GMimeStream *stream);
 static int stream_reset (GMimeStream *stream);
 static gint64 stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence);
 static gint64 stream_tell (GMimeStream *stream);
-static ssize_t stream_length (GMimeStream *stream);
+static gint64 stream_length (GMimeStream *stream);
 static GMimeStream *stream_substream (GMimeStream *stream, gint64 start, gint64 end);
 
 
@@ -151,7 +151,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	
 	/* don't allow our caller to read past the end of the stream */
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (gint64) len);
+		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	if (!(current = cat->current))
 		return -1;
@@ -196,7 +196,7 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	
 	/* don't allow our caller to write past the end of the stream */
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (gint64) len);
+		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	if (!(current = cat->current))
 		return -1;
@@ -329,8 +329,7 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 {
 	GMimeStreamCat *cat = (GMimeStreamCat *) stream;
 	struct _cat_node *current, *n;
-	gint64 real, off;
-	ssize_t len;
+	gint64 real, off, len;
 	
 	d(fprintf (stderr, "GMimeStreamCat::stream_seek (%p, %ld, %d)\n",
 		   stream, offset, whence));
@@ -432,7 +431,7 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 						return -1;
 				}
 				
-				d(fprintf (stderr, "real = %ld, stream[%d] len = %ld\n",
+				d(fprintf (stderr, "real = %lld, stream[%d] len = %lld\n",
 					   real, current->id, len));
 				
 				if ((real + len) > offset) {
@@ -528,11 +527,11 @@ stream_tell (GMimeStream *stream)
 	return stream->position;
 }
 
-static ssize_t
+static gint64
 stream_length (GMimeStream *stream)
 {
 	GMimeStreamCat *cat = GMIME_STREAM_CAT (stream);
-	ssize_t len, total = 0;
+	gint64 len, total = 0;
 	struct _cat_node *n;
 	
 	if (stream->bound_end != -1)
@@ -564,7 +563,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 	gint64 offset = 0, subend = 0;
 	GMimeStream *substream;
 	struct _cat_node *n;
-	ssize_t len;
+	gint64 len;
 	
 	d(fprintf (stderr, "GMimeStreamCat::substream (%p, %ld, %ld)\n", stream, start, end));
 	
