@@ -23,6 +23,12 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
+
+#ifdef G_OS_WIN32
+#include <fcntl.h>
+#include <io.h>
+#endif
 #include <errno.h>
 
 #include "gmime-stream-file.h"
@@ -54,7 +60,7 @@ static gboolean stream_eos (GMimeStream *stream);
 static int stream_reset (GMimeStream *stream);
 static gint64 stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence);
 static gint64 stream_tell (GMimeStream *stream);
-static ssize_t stream_length (GMimeStream *stream);
+static gint64 stream_length (GMimeStream *stream);
 static GMimeStream *stream_substream (GMimeStream *stream, gint64 start, gint64 end);
 
 
@@ -144,25 +150,13 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	}
 	
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (gint64) len);
+		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
 	fseek (fstream->fp, (long) stream->position, SEEK_SET);
 	
-	if ((nread = fread (buf, 1, len, fstream->fp)) > 0) {
-#ifdef G_OS_WIN32
-		/* fucking Windows... sigh. Since it might decide to translate \r\n into \n,
-		 * we need to query the real position rather than using simple math. */
-		long pos;
-		
-		if ((pos = ftell (fstream->fp)) == -1)
-			stream->position += nread;
-		else
-			stream->position = pos;
-#else
+	if ((nread = fread (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nread;
-#endif
-	}
 	
 	return (ssize_t) nread;
 }
@@ -184,25 +178,13 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	}
 	
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (gint64) len);
+		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
 	fseek (fstream->fp, (long) stream->position, SEEK_SET);
 
-	if ((nwritten = fwrite (buf, 1, len, fstream->fp)) > 0) {
-#ifdef G_OS_WIN32
-		/* fucking Windows... sigh. Since it might decide to translate \n into \r\n,
-		 * we need to query the real position rather than using simple math. */
-		long pos;
-		
-		if ((pos = ftell (fstream->fp)) == -1)
-			stream->position += nwritten;
-		else
-			stream->position = pos;
-#else
+	if ((nwritten = fwrite (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nwritten;
-#endif
-	}
 	
 	return (ssize_t) nwritten;
 }
@@ -331,7 +313,7 @@ stream_tell (GMimeStream *stream)
 	return stream->position;
 }
 
-static ssize_t
+static gint64
 stream_length (GMimeStream *stream)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
@@ -388,6 +370,10 @@ g_mime_stream_file_new (FILE *fp)
 	GMimeStreamFile *fstream;
 	gint64 start;
 	
+#ifdef G_OS_WIN32
+	_setmode (_fileno (fp), O_BINARY);
+#endif
+	
 	if ((start = ftell (fp)) == -1)
 		start = 0;
 	
@@ -419,6 +405,10 @@ g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
+#ifdef G_OS_WIN32
+	_setmode (_fileno (fp), O_BINARY);
+#endif
+	
 	fstream = g_object_newv (GMIME_TYPE_STREAM_FILE, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fstream), start, end);
 	fstream->owner = TRUE;
