@@ -27,14 +27,9 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 #include <fcntl.h>
 #include <errno.h>
-#ifdef G_OS_WIN32
-#include <io.h>
-#endif
 
 #include "gmime-stream-fs.h"
 
@@ -72,7 +67,7 @@ static gboolean stream_eos (GMimeStream *stream);
 static int stream_reset (GMimeStream *stream);
 static gint64 stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence);
 static gint64 stream_tell (GMimeStream *stream);
-static ssize_t stream_length (GMimeStream *stream);
+static gint64 stream_length (GMimeStream *stream);
 static GMimeStream *stream_substream (GMimeStream *stream, gint64 start, gint64 end);
 
 
@@ -162,7 +157,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	}
 	
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (gint64) len);
+		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
 	lseek (fs->fd, (off_t) stream->position, SEEK_SET);
@@ -172,18 +167,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	} while (nread == -1 && errno == EINTR);
 	
 	if (nread > 0) {
-#ifdef G_OS_WIN32
-		/* fucking Windows... sigh. Since it might decide to translate \r\n into \n,
-		 * we need to query the real position rather than using simple math. */
-		off_t pos;
-		
-		if ((pos = lseek (fs->fd, 0, SEEK_CUR)) == -1)
-			stream->position += nread;
-		else
-			stream->position = pos;
-#else
 		stream->position += nread;
-#endif
 	} else if (nread == 0) {
 		fs->eos = TRUE;
 	}
@@ -209,7 +193,7 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	}
 	
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (gint64) len);
+		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
 	lseek (fs->fd, (off_t) stream->position, SEEK_SET);
@@ -227,18 +211,7 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 		fs->eos = TRUE;
 	
 	if (nwritten > 0) {
-#ifdef G_OS_WIN32
-		/* fucking Windows... sigh. Since it might decide to translate \n into \r\n,
-		 * we need to query the real position rather than using simple math. */
-		off_t pos;
-		
-		if ((pos = lseek (fs->fd, 0, SEEK_CUR)) == -1)
-			stream->position += nwritten;
-		else
-			stream->position = pos;
-#else
 		stream->position += nwritten;
-#endif
 	} else if (n == -1) {
 		/* error and nothing written */
 		return -1;
@@ -389,7 +362,7 @@ stream_tell (GMimeStream *stream)
 	return stream->position;
 }
 
-static ssize_t
+static gint64
 stream_length (GMimeStream *stream)
 {
 	GMimeStreamFs *fs = (GMimeStreamFs *) stream;
@@ -431,7 +404,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 
 /**
  * g_mime_stream_fs_new:
- * @fd: file descriptor
+ * @fd: a file descriptor
  *
  * Creates a new #GMimeStreamFs object around @fd.
  *
@@ -443,6 +416,10 @@ g_mime_stream_fs_new (int fd)
 	GMimeStreamFs *fs;
 	gint64 start;
 	
+#ifdef G_OS_WIN32
+	_setmode (fd, O_BINARY);
+#endif
+	
 	if ((start = lseek (fd, (off_t) 0, SEEK_CUR)) == -1)
 		start = 0;
 	
@@ -472,6 +449,10 @@ g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end)
 {
 	GMimeStreamFs *fs;
 	
+#ifdef G_OS_WIN32
+	_setmode (fd, O_BINARY);
+#endif
+	
 	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
 	fs->owner = TRUE;
