@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ int main (int argc, char **argv)
 	GMimeFilterBest *best;
 	int fd;
 	
-	if ((fd = open (argv[1], O_RDONLY)) == -1)
+	if ((fd = open (argv[1], O_RDONLY, 0)) == -1)
 		return 0;
 	
 	g_mime_init (0);
@@ -57,13 +57,13 @@ int main (int argc, char **argv)
 	printf ("\tbest charset: %s\n", g_mime_filter_best_charset (best));
 	printf ("\tbest encoding (7bit): %s\n",
 		g_mime_content_encoding_to_string (
-			g_mime_filter_best_encoding (best, GMIME_BEST_ENCODING_7BIT)));
+			g_mime_filter_best_encoding (best, GMIME_ENCODING_CONSTRAINT_7BIT)));
 	printf ("\tbest encoding (8bit): %s\n",
 		g_mime_content_encoding_to_string (
-			g_mime_filter_best_encoding (best, GMIME_BEST_ENCODING_8BIT)));
+			g_mime_filter_best_encoding (best, GMIME_ENCODING_CONSTRAINT_8BIT)));
 	printf ("\tbest encoding (binary): %s\n",
 		g_mime_content_encoding_to_string (
-			g_mime_filter_best_encoding (best, GMIME_BEST_ENCODING_BINARY)));
+			g_mime_filter_best_encoding (best, GMIME_ENCODING_CONSTRAINT_BINARY)));
 	
 	g_object_unref (istream);
 	g_object_unref (null);
